/*
 * @file asort.c
 *
 * Example for using qsort(3), slightly adapted from the Linux
 * qsort(3) man page. It simply sorts the argument vector passed to
 * the main function.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int
cmpstringp(const void *p1, const void *p2)
{
    /* The actual arguments to this function are "pointers to
       pointers to char", but strcmp(3) arguments are "pointers
       to char", hence the following cast plus dereference */
    
    return strcmp(* (char * const *) p1, * (char * const *) p2);
}

int
cmpstringlenp(const void *p1, const void *p2)
{
    char *s1 = * (char **) p1;
    char *s2 = * (char **) p2;
    
    int l1 = strlen(s1);
    int l2 = strlen(s2);

    if (l1 < l2) {
	return -1;
    }
    if (l1 > l2) {
	return 1;
    }

    return strcmp(s1, s2);
}

int
main(int argc, char *argv[])
{
    int j;
    
    if (argc < 2) {
	fprintf(stderr, "Usage: %s <string>...\n", argv[0]);
	exit(EXIT_FAILURE);
    }
    
    qsort(&argv[1], argc - 1, sizeof(char *), cmpstringp);
    qsort(&argv[1], argc - 1, sizeof(char *), cmpstringlenp);
    
    for (j = 1; j < argc; j++) {
	printf("%s\n", argv[j]);
    }
    
    return EXIT_SUCCESS;
}
